/* $Id: crires_util_convert.c,v 1.15 2011/02/09 10:16:18 yjung Exp $
 *
 * This file is part of the crires Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: yjung $
 * $Date: 2011/02/09 10:16:18 $
 * $Revision: 1.15 $
 * $Name: crire-2_1_1 $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/

#include "crires_recipe.h"

/*-----------------------------------------------------------------------------
                                Define
 -----------------------------------------------------------------------------*/

#define RECIPE_STRING "crires_util_convert"

/*-----------------------------------------------------------------------------
                            Functions prototypes
 -----------------------------------------------------------------------------*/

static int crires_util_convert_save(cpl_imagelist **, 
        const cpl_parameterlist *, cpl_frameset *) ;

static char crires_util_convert_description[] = 
"This recipe accepts 3 files (produced by crires_spec_detlin):\n"
"First parameter:   the image with the A coefficients.\n"
"Second parameter:  the image with the B coefficients.\n"
"Third parameter:   the image with the C coefficients.\n" 
"\n"
"This recipe produces 1 file:\n"
"First parameter:   the file with the detlin coefficients.\n"
"                   (PRO TYPE = "CRIRES_PROTYPE_COEFFS_CUBE")\n" ;

CRIRES_RECIPE_DEFINE(crires_util_convert,
        0,
        "Convert the detlin coefficients",
        crires_util_convert_description) ;

/*-----------------------------------------------------------------------------
                                Functions code
 -----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
  @brief    Interpret the command line options and execute the data processing
  @param    frameset   the frames list
  @param    parlist     the parameters list
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int crires_util_convert(
        cpl_frameset            *   frameset,
        const cpl_parameterlist *   parlist)
{
    cpl_imagelist       **  coeffs ;
    cpl_frame           *   fr[3] ;
    cpl_image           *   cur_ima ;
    int                     i, j ;
    
    /* Retrieve input parameters */

    /* Identify the RAW and CALIB frames in the input frameset */
    if (crires_dfs_set_groups(frameset, NULL)) {
        cpl_msg_error(__func__, "Cannot identify RAW and CALIB frames") ;
        return -1 ;
    }
    
    /* Check that we have 3 files in input */
    if (cpl_frameset_get_size(frameset) != 3) {
        cpl_msg_error(__func__, "Expects 3 file in input") ;
        return -1 ;
    }
    fr[0] = cpl_frameset_get_frame(frameset, 0);
    fr[1] = cpl_frameset_get_frame(frameset, 1);
    fr[2] = cpl_frameset_get_frame(frameset, 2);

    /* Create the image lists */
    coeffs = cpl_malloc(CRIRES_NB_DETECTORS * sizeof(cpl_imagelist*)) ;
    for (i=0 ; i<CRIRES_NB_DETECTORS; i++) 
        coeffs[i] = cpl_imagelist_new() ;

    /* Loop on the detectors */
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        cpl_msg_info(__func__, "Get the coefficients for chip number %d", i+1) ;
        /* Coefficient A <- fr[0] */
        if ((cur_ima = cpl_image_load(cpl_frame_get_filename(fr[0]), 
                        CPL_TYPE_FLOAT, 0, i+1)) == NULL) {
            cpl_msg_error(__func__, "Cannot load ext %d", i+1) ;
            for (j=0 ; j<CRIRES_NB_DETECTORS; j++) 
                cpl_imagelist_delete(coeffs[j]) ;
            cpl_free(coeffs) ;
            return -1 ;
        }
        cpl_imagelist_set(coeffs[i], cur_ima, 0) ;

        /* Coefficient B  <- fr[1] */
        if ((cur_ima = cpl_image_load(cpl_frame_get_filename(fr[1]), 
                        CPL_TYPE_FLOAT, 0, i+1)) == NULL) {
            cpl_msg_error(__func__, "Cannot load ext %d", i+1) ;
            for (j=0 ; j<CRIRES_NB_DETECTORS; j++) 
                cpl_imagelist_delete(coeffs[j]) ;
            cpl_free(coeffs) ;
            return -1 ;
        }
        cpl_imagelist_set(coeffs[i], cur_ima, 1) ;

        /* Coefficient C <- fr[2] */
        if ((cur_ima = cpl_image_load(cpl_frame_get_filename(fr[2]), 
                        CPL_TYPE_FLOAT, 0, i+1)) == NULL) {
            cpl_msg_error(__func__, "Cannot load ext %d", i+1) ;
            for (j=0 ; j<CRIRES_NB_DETECTORS ; j++) 
                cpl_imagelist_delete(coeffs[j]) ;
            cpl_free(coeffs) ;
            return -1 ;
        }
        cpl_imagelist_set(coeffs[i], cur_ima, 2) ;
    }
    
    /* Save the products */
    cpl_msg_info(__func__, "Save the products") ;
    cpl_msg_indent_more() ;
    if (crires_util_convert_save(coeffs, parlist, frameset)==-1) {
        cpl_msg_error(__func__, "Cannot save the products") ;
        for (j=0 ; j<CRIRES_NB_DETECTORS; j++) cpl_imagelist_delete(coeffs[j]) ;
        cpl_free(coeffs) ;
        cpl_msg_indent_less() ;
        return -1 ;
    }
    cpl_msg_indent_less() ;

    /* Return */
    for (i=0 ; i<CRIRES_NB_DETECTORS; i++) cpl_imagelist_delete(coeffs[i]) ;
    cpl_free(coeffs) ;
    if (cpl_error_get_code()) return -1 ;
    else return 0 ;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Save the different products of the recipe
  @param    coeffs      the 4 detectors coefficients imagelists
  @param    parlist     the input list of parameters
  @param    set         the input frame set
  @return   0 if everything is ok, -1 otherwise
 */
/*----------------------------------------------------------------------------*/
static int crires_util_convert_save(
        cpl_imagelist           **  coeffs,
        const cpl_parameterlist *   parlist,
        cpl_frameset            *   set)
{
    const char          *   recipe_name = "crires_util_convert" ;
    cpl_propertylist    *   pro_list ;
    cpl_propertylist    *   qc_ext_list ;
    char                    sval[16] ;
    char                *   filename ;
    int                     i ;

    /* Output name */
    filename = cpl_sprintf("crires_util_convert.fits") ;

    /* PRO keys */
    pro_list = cpl_propertylist_new() ;
    cpl_propertylist_append_string(pro_list, CPL_DFS_PRO_CATG, 
            CRIRES_CALPRO_COEFFS_CUBE) ;
    cpl_propertylist_append_string(pro_list, CPL_DFS_PRO_TYPE,
             CRIRES_PROTYPE_COEFFS_CUBE) ;

    /* File with extensions */
    if (cpl_dfs_save_image(set, NULL, parlist, set, NULL, NULL,
                CPL_BPP_IEEE_FLOAT, recipe_name, pro_list, NULL, 
                PACKAGE "/" PACKAGE_VERSION, filename) != CPL_ERROR_NONE) {
        cpl_msg_error(__func__, "Cannot save the empty primary HDU") ;
        cpl_propertylist_delete(pro_list) ;
        cpl_free(filename) ;
        return -1 ;
    }
    /* Delete PRO LIST */
    cpl_propertylist_delete(pro_list) ;

    /* Save the extensions */
    for (i=0 ; i<CRIRES_NB_DETECTORS ; i++) {
        /* Extension header */
        qc_ext_list = cpl_propertylist_new() ;
        sprintf(sval, "CHIP%d.INT1", i+1) ;
        cpl_propertylist_prepend_string(qc_ext_list, "EXTNAME", sval) ;
        cpl_imagelist_save(coeffs[i],
                filename, CPL_BPP_IEEE_FLOAT, qc_ext_list, CPL_IO_EXTEND) ;
        cpl_propertylist_delete(qc_ext_list) ;
    }
    cpl_free(filename) ;
    return  0;
}

